const async = require('async');
const request = require('request');
const database = require('./database');
const handle = require('./handle');

function reports(emailId, authtoken, callback) {
  async.waterfall([
    (callback) => {
      request.get({
        baseUrl: 'https://reportsapi.zoho.com',
        uri: `/api/${emailId}`,
        headers: {
          'Content-Type': 'application/x-www-form-urlencoded'
        },
        qs: {
          'ZOHO_ACTION': 'DATABASEMETADATA',
          'ZOHO_METADATA': 'ZOHO_CATALOG_LIST',
          'ZOHO_API_VERSION': '1.0',
          'ZOHO_OUTPUT_FORMAT': 'JSON',
          'ZOHO_ERROR_FORMAT': 'JSON',
          'authtoken': authtoken
        }
      }, (err, res, body) => {
        handle(err, res, body, callback);
      });
    },
    (result, callback) => {
      callback(null, result.map(row => row.tableCat));
    },
    (dbnames, callback) => {
      // Return reports context
      callback(null, {
        // Get list of databases in the account
        databases: callback => callback(null, dbnames),
        // Get database
        database: (dbname, callback) => {
          if (dbnames.indexOf(dbname) > 0) {
            return database(emailId, authtoken, dbname, callback);
          } else {
            return callback(new Error(`Database ${dbname} does not exist in account ${emailId}`));
          }
        },
        // Quick access to table
        table: (dbname, tname, callback) => {
          if (dbnames.indexOf(dbname) < 0) {
            return callback(new Error(`Database ${dbname} does not exist in account ${emailId}`));
          }

          database(emailId, authtoken, dbname, (err, db) => {
            if (err) {
              return callback(err);
            }

            db.table(tname, (err, t) => {
              if (err) {
                return callback(err);
              }

              callback(null, t);
            });
          });
        }
      });
    }
  ], (err, reports) => {
    if (err) {
      return callback(err);
    }

    callback(null, reports);
  });
}

module.exports = reports;
